#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>
#include "loadpic.h"

PPMImage *LoadBMP(const char *filename)
{
    FILE          *fin;
    PPMImage *result;
    int           bpp=0;
    unsigned char bmpHeader[54]={0x42, 0x4d,        // magicke cislo souboru BMP
                        0x00, 0x00, 0x00, 0x00,     // velikost souboru
                        0x00, 0x00, 0x00, 0x00,     // rezervovano, vzdy nastaveno na nula
                        0x36, 0x04, 0x00, 0x00,     // data offset=54
                        0x28, 0x00, 0x00, 0x00,     // velikost hlavicky=40
                        0x00, 0x00, 0x00, 0x00,     // sirka obrazku v pixelech=?
                        0x00, 0x00, 0x00, 0x00,     // vyska obrazku v pixelech=?
                        0x01, 0x00,                 // pocet bitovych rovin=1
                        0x08, 0x00,                 // pocet bitu na pixel=24
                        0x00, 0x00, 0x00, 0x00,     // metoda komprimace=nic
                        0x00, 0x00, 0x00, 0x00,     // velikost bitmapy
                        0x00, 0x00, 0x00, 0x00,     // pocet pixelu na metr v horizontalnim smeru
                        0x00, 0x00, 0x00, 0x00,     // pocet pixelu na metr ve vertikalnim smeru
                        0x00, 0x00, 0x00, 0x00,     // pocet pouzitych barev
                        0x00, 0x00, 0x00, 0x00,     // pocet dulezitych barev
    };
    if (!filename) return NULL;
    fin=fopen(filename, "rb");
    if (!fin) return NULL;                            // otevreni souboru se nezdarilo
    result = (PPMImage *) malloc(sizeof(PPMImage));
    if (fread(bmpHeader, 54, 1, fin)!=1) return NULL; // nacist hlavicku BMP souboru
    memcpy(&result->sizeX, bmpHeader+18, 4);                // sirka obrazku v pixelech
    memcpy(&result->sizeY, bmpHeader+22, 4);               // vyska obrazku v pixelech
    memcpy(&bpp, bmpHeader+28, 2);                  // pocet bitu na pixel
    if (bpp!=24) return NULL;
    result->data = (GLubyte *) malloc(3 * result->sizeX * result->sizeY);
    if (fread(result->data, 3 * result->sizeX * result->sizeY, sizeof(unsigned char), fin)!=1) return NULL;// nacteni rastrovych dat
    fclose(fin);
                                  
    return result;    
}


PPMImage *LoadPPM(const char *filename)
{
    char buff[16];
    PPMImage *result;
    FILE *fp;
    int maxval;

    fp = fopen(filename, "rb");
    if (fp==NULL) return NULL;
    if (!fgets(buff, sizeof(buff), fp)) return NULL;
    if (buff[0] != 'P' || buff[1] != '6') return NULL;
    result = (PPMImage *) malloc(sizeof(PPMImage));
    if (!result) return NULL;
    if (fscanf(fp, "%d %d", &result->sizeX, &result->sizeY) != 2) return NULL;
    if (fscanf(fp, "%d", &maxval) != 1) return NULL;

    while (fgetc(fp) != '\n')
	  ;
	  
    result->data = (GLubyte *) malloc(3 * result->sizeX * result->sizeY);
    if (!result) return NULL;
    if (fread(result->data, 3 * result->sizeX, result->sizeY, fp) != result->sizeY) return NULL;

    fclose(fp);
    return result;
}
