//---------------------------------------------------------------------
// Strelnice - semestralni prace na 36ZPG
// CVUT FEL VT
//
// (c)2005 programmed by GS ( goldsoft@post.cz )
//
// graphics: kO.dLL
// sounds: JakuBLB
//
// THX to: Nate Robins, Pavel Tisnovsky
//---------------------------------------------------------------------

#if defined(_WIN32)
#include <windows.h>
#else
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <GL/glut.h>
#include "glm.h"

#include "utils.h"

GLfloat zx=0,zy=0,zz=0;
GLfloat matice[4][4];
GLfloat matice2[4][4];

GLint cas=0;
const char *zvukStrela;

float materialAmbient[]={0.3, 0.2, 0.2, 1.0}; // ambientni slozka materialu
float materialDiffuse[]={0.9, 0.1, 0.1, 1.0}; // difuzni slozka materialu
float materialSpecular[]={0.0, 1.0, 1.0, 0.5}; // barva odlesku
float lightAmbient[]= {0.5, 0.5, 0.5, 1.0}; // ambientni slozka svetla
float lightDiffuse[]= {0.9, 0.9, 0.9, 1.0}; // difuzni slozka svetla
float lightSpecular[]={0.0, 0.0, 0.0, 1.0}; // barva odlesku
float lightPosition[]={1.0, 1.0, 1.0, 0.0}; // definice pozice svetla
GLfloat fogColor[4]={0.5, 0.5, 0.5, 1.0};

TInfo info; // extern

TClovek clovek;
TZbran zbran;
TTerc terc;
TStrelnice strelnice;

//---------------------------------------------------------------------
// zvuk nacteni
//---------------------------------------------------------------------
long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}

const char *LoadSound(const char *filename)
{
    FILE          *fin;
    char *result;

    if (!filename) return NULL;
    fin=fopen(filename, "rb");
    if (!fin) return NULL;                            // otevreni souboru se nezdarilo
    
    long size=filesize(fin);    
    
    result = (char *) malloc(size);
    
    if (fread(result,size,1, fin)!=1) return NULL;
    fclose(fin);
                                  
    return result;    
}

//---------------------------------------------------------------------
// Tato funkce provede zakladni nastaveni vykreslovani teles
//---------------------------------------------------------------------
void init(void)
{
    glClearColor(0.5,0.5,0.5,1.0);          // barva pro mazani color-bufferu
    glShadeModel(GL_SMOOTH);                // nastaveni Gouraudova stinovani
    glClearDepth(1.0f);                     // barva pro mazani Z-bufferu
    glClearStencil(0xff);                   // nastaveni mazaci hodnoty stencil bufferu
    glEnable(GL_DEPTH_TEST);                // nastaveni funkce pro testovani hodnot v Z-bufferu
    glDepthFunc(GL_LEQUAL);                   // nastaveni porovnavaci funkce pro Z-buffer
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);// vylepseni zobrazovani textur    
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); // stav vykresleni vyplnenych polygonu
    glDisable(GL_CULL_FACE);                // zadne hrany ani steny se nebudou odstranovat
    glPointSize(2.0);                       // velikost vykreslovanych bodu
    glLineWidth(1.0);                       // sirka vykreslovanych car
    glEnable(GL_POINT_SMOOTH);              // povoleni antialiasingu bodu
    glEnable(GL_LINE_SMOOTH);               // povoleni antialiasingu car
    glLightfv(GL_LIGHT0,GL_AMBIENT,lightAmbient); // nastaveni parametru svetla
    glLightfv(GL_LIGHT0,GL_DIFFUSE,lightDiffuse);
    glLightfv(GL_LIGHT0,GL_POSITION,lightPosition);
    glEnable(GL_LIGHTING);                  // povoleni osvetleni
    glEnable(GL_LIGHT0);                    // zapnuti prvniho svetla
    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,materialAmbient);// nastaveni vlastnosti materialu
    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,materialDiffuse);
    glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,materialSpecular);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20.0);
    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);         // nemenit hodnoty ve stencil bufferu
    glDisable(GL_STENCIL_TEST);
    glFogfv(GL_FOG_COLOR, fogColor);    
    glFogf(GL_FOG_DENSITY, 0.135);
    glFogi(GL_FOG_MODE, GL_LINEAR);
    glHint(GL_FOG_HINT, GL_DONT_CARE);
    glFogf(GL_FOG_START, 150.0);
    glFogf(GL_FOG_END, 1000.0);
    glEnable(GL_FOG);
    initTextury();
    zvukStrela=LoadSound("data/scout.wav");
}



//---------------------------------------------------------------------
// Nastaveni souradneho systemu v zavislosti na velikosti okna
//---------------------------------------------------------------------
void onResize(int w, int h)
{
    info.windowCenterX=w/2;
    info.windowCenterY=h/2;
    glViewport(0, 0, w, h);                 // viditelna oblast
    glMatrixMode(GL_PROJECTION);            // zmena projekcni matice
    glLoadIdentity();                       // nastaveni perspektivni projekce
    gluPerspective(clovek.fov,(double)info.windowCenterX/(double)info.windowCenterY,clovek.nearPlane,clovek.farPlane);
    glMatrixMode(GL_MODELVIEW);             // zmena modelove matice
}

//---------------------------------------------------------------------
// Tato funkce je volana pri kazdem prekresleni okna
//---------------------------------------------------------------------
void onDisplay(void)
{
    static char s[256];
    static int frames = 0;
    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT); // vymazani barvoveho a hloubkoveho bufferu
    
    glMatrixMode(GL_PROJECTION);                    // projekcni matice
    
    glLoadIdentity();
    gluPerspective(clovek.fov,(GLdouble)info.windowCenterX/(GLdouble)info.windowCenterY,clovek.nearPlane,clovek.farPlane);
    
    glMatrixMode(GL_MODELVIEW);             // bude se menit modelova matice
    
    GLfloat alfa=(GLfloat)clovek.posAlfaD-3;
    GLfloat beta=(GLfloat)clovek.posBetaD-2;
    
    glLoadIdentity();
    glRotatef(beta,1,0.0,0.0);
    glGetFloatv(GL_MODELVIEW_MATRIX,*matice);
    
    glLoadIdentity();
    glRotatef(alfa,0.0,0.0,-1);
//    glGetFloatv(GL_MODELVIEW_MATRIX,*matice2);
    
    glMultMatrixf(*matice);
    glGetFloatv(GL_MODELVIEW_MATRIX,*matice);
    
/*    
     for (int i=0;i<3;i++) 
       for (int ii=0;i<3;i++) 
         if (i!=ii) matice[i][ii]=matice[i][ii]+matice2[i][ii];
         
    glGetFloatv(GL_MODELVIEW_MATRIX,*matice);
*/    
    
    if (info.textured) 
    {
      glEnable(GL_TEXTURE_2D);                        // povoleni texturovani
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_LINEAR); // opakovani textury
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_LINEAR);
    }
    else glDisable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);    
    
    glLoadIdentity();                       // nahrat jednotkovou matici
    gluLookAt(clovek.eyeX,clovek.eyeY,clovek.eyeZ,clovek.centerX,clovek.centerY,clovek.centerZ,0,0,1);    // nastaveni pozice a orientace kamery    

    strelnice.kresli();
    
    glPushMatrix();
    
    glTranslatef(clovek.posX,clovek.posY,clovek.posZ);
    glMultMatrixf(*matice);
    
    zbran.kresli();
    glPopMatrix();
    glTranslatef(-25,197,26);
    glEnable(GL_STENCIL_TEST);
    for (int ii=0;ii<3;ii++)
    {
      for (int i=0;i<6;i++) {
        if (info.terce[ii][i]==1) {
          glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
          glStencilFunc(GL_ALWAYS,i+ii*6,0x01);           // kreslit do stencil bufferu    
          terc.kresli();
          glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);         // nemenit hodnoty ve stencil bufferu
        }
        else if (info.terce[ii][i]==2) terc.kresli(false);        
        glTranslatef(10,0,0);
      }
      glTranslatef(-6*10,0,-14);
    }
    glDisable(GL_STENCIL_TEST);
    glDisable(GL_TEXTURE_2D);
    if (info.showfps) {
      /* spit out frame rate. */
      frames++;
      if (frames > 5) {
        sprintf(s, "%g fps", frames/elapsed());
        frames = 0;
      }
	    printText(5, 5, s);                 
	    /*
    	int height = glutGet(GLUT_WINDOW_HEIGHT);
      // sprintf(s, "\neyeX: %f\neyeY: %f\neyeZ: %f\nposX: %f\nposY: %f\nkukX: %f\nkukY: %f\nkukZ: %f\nalfa: %f\nbeta: %f\nsensi: %f\n", \
      // clovek.eyeX,clovek.eyeY,clovek.eyeZ,clovek.posX,clovek.posY,clovek.centerX,clovek.centerY,clovek.centerZ,clovek.posAlfa,clovek.posBeta,clovek.sensi);
      // sprintf(s,"\ntmpi1:%d\ntmpi2:%d\ntmpf1:%f\ntmpf2:%f",info.tmpi1,info.tmpi2,info.tmpf1,info.tmpf2);
      sprintf(s,"\nx:%f\ny:%f\nz:%f",zx,zy,zz);      
      // sprintf(s,"\nx:%f\ny:%f\nz:%f",clovek.posX,clovek.posY,clovek.posZ);
	    printText(5, height-(5+18*1), s);
	    */
    }
    
    glFlush();                                      // provedeni vsech prikazu
    glutSwapBuffers();                              // a prohozeni bufferu
}



//---------------------------------------------------------------------
// Tato funkce je volana pri stlaceni ASCII klavesy
//---------------------------------------------------------------------
void onKeyboard(unsigned char key, int x, int y)
{
    key=(key>'A' && key<='Z') ? key+'a'-'A':key; // prevod na mala pismena
    GLfloat krok=1;

    switch (key) {                          // rozeskok podle stlacene klavesy
      case 'a':
        clovek.jdiZ();
        break;
      case 'z':
        clovek.jdiZ(false);
        break;
      case 's':
        zx=zx+krok;
        break;
      case 'x':
        zx=zx-krok;
        break;
      case 'd':
        zy=zy+krok;
        break;
      case 'c':
        zy=zy-krok;
        break;
      case 'r':
        zz=zz+krok;
        break;
      case 'v':
        zz=zz-krok;
        break;
      case 'f':
        toggleFullScreen();
        break;
      case 13:
        initNewGame();
        glutPostRedisplay();
        break;
      case 27:                            // klavesa Escape
        exit(0);                        // ukonceni programu
        break;
      default:
        break;
    }
    glutPostRedisplay();                    // prekresleni sceny    
}

void onKeyboardSpec(int key, int x, int y)
{
    int modifiers=glutGetModifiers();       // ziskat stav klaves ALT, CTRL a SHIFT 
    switch (key) {                          // rozeskok podle stlacene klavesy
        case GLUT_KEY_LEFT:
          if (modifiers & GLUT_ACTIVE_SHIFT) clovek.jdiX(true,true);
          else clovek.jdiX();
          break;
        case GLUT_KEY_RIGHT:
          if (modifiers & GLUT_ACTIVE_SHIFT) clovek.jdiX(false,true);          
          else clovek.jdiX(false);
          break;
        case GLUT_KEY_UP:
          if (modifiers & GLUT_ACTIVE_SHIFT) clovek.jdiY(true,true);
          else clovek.jdiY();
          break;
        case GLUT_KEY_DOWN:
          if (modifiers & GLUT_ACTIVE_SHIFT) clovek.jdiY(false,true);
          else clovek.jdiY(false);
          break;
        default:
          break;
    }
    glutPostRedisplay();                    // prekresleni sceny    
}

//---------------------------------------------------------------------
// Tato funkce je volana pri zmene stavu tlacitek mysi
//---------------------------------------------------------------------

void zvukStrely(void) {
  #if defined(_WIN32)
  // PlaySound("data/scout.wav",NULL,SND_ASYNC|SND_FILENAME);
  PlaySound(zvukStrela,NULL,SND_ASYNC|SND_MEMORY|SND_NODEFAULT);
  #endif
}
void onMouseClick(int button, int state, int x, int y)
{
  GLint *p=new GLint[1];
  if (button==GLUT_LEFT_BUTTON) {        // prave tlacitko aktivuje posun od pozorovatele
    if (state==GLUT_DOWN) {             // pri stlaceni
      zvukStrely();
      glReadPixels(x,y,1,1,GL_STENCIL_INDEX,GL_INT,p);
      info.tmpi1=*p;
      int ii=*p/6;
      int i=*p%6;
      if (*p<=18 && *p>=0 && info.terce[ii][i]) info.terce[ii][i]=2;
    }
  }
    glutPostRedisplay();                    // prekresleni sceny
}

//---------------------------------------------------------------------
// Tato funkce je volana pri idlu
//---------------------------------------------------------------------
void onIdle()
{
  glutPostRedisplay ();                     // Pekreslen okna
}

//---------------------------------------------------------------------
// Tato funkce je volana pri posunu mysi bez stlaceneho tlacitka
//---------------------------------------------------------------------
void onMouseMove(int x, int y)
{
  if (!info.mouseCheck) return;
  static int tmpX,tmpY;
  tmpX=x-info.windowCenterX;
  tmpY=y-info.windowCenterY;
  if (tmpX>10 || tmpX<10)
  {
    1==1;
    2==2;
  }
  if (tmpX<0) clovek.otocX(true,tmpX*-1);
  else if (tmpX>0) clovek.otocX(false,tmpX);
  if (tmpY<0) clovek.otocZ(true,tmpY*-1);
  else if (tmpY>0) clovek.otocZ(false,tmpY);
  if (tmpX>5 || tmpX<-5 || tmpY>5 || tmpY<-5) glutWarpPointer(info.windowCenterX,info.windowCenterY);    
  glutPostRedisplay();                    // prekreslit scenu
}

//---------------------------------------------------------------------
// Callback funkce zavolana pri tiku casovace kazdych 10ms
//---------------------------------------------------------------------
void onTimer(int timer)
{
  cas=cas+1;
  glutTimerFunc(10, onTimer, timer);
}

//---------------------------------------------------------------------
// Hlavni funkce konzolove aplikace
//---------------------------------------------------------------------
int main(int argc, char *argv[])
{
    info.fullscreen=false;
    info.showfps=false;
    info.textured=true;
    info.mouseCheck=true;
    info.filled=true;
    info.clip=true;
    zbran.visible=true;
    initNewGame();
    glutInit(&argc,argv);                  // inicializace knihovny GLUT
    glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE|GLUT_DEPTH|GLUT_STENCIL); // graficky mod okna
    glutInitWindowSize(500,500);           // pocatecni velikost okna
    glutInitWindowPosition(50,50);         // pocatecni pozice okna
    glutCreateWindow("GS Strelnice");       // vytvoreni okna pro kresleni
    glutReshapeWindow(500,500);            // nastaveni velikosti okna
    glutDisplayFunc(onDisplay);             // registrace funkce volane pri prekreslovani okna
    glutReshapeFunc(onResize);              // registrace funkce volane pri zmene velikosti okna
    glutKeyboardFunc(onKeyboard);           // registrace funkce volane pri stlaceni ASCII klavesy
    glutSpecialFunc(onKeyboardSpec);        // registrace funkce volane pri stlaceni specialni klavesy (sipky)
    glutMouseFunc(onMouseClick);            // registrace funkce volane pri stlaceni nebo pusteni tlacitka mysi
    glutPassiveMotionFunc(onMouseMove);     // registrace funkce volane pri posunu mysi bez stlaceneho tlacitka
    glutIdleFunc(onIdle);                   // registrace funkce volane pri idlu
    glutSetCursor(GLUT_CURSOR_NONE);        // neviditelny kurzor
    info.windowCenterX=glutGet(GLUT_WINDOW_WIDTH)/2;
    info.windowCenterY=glutGet(GLUT_WINDOW_HEIGHT)/2;
    glutWarpPointer(info.windowCenterX,info.windowCenterY);
    init();                                 // inicializace vykreslovani
    createMenu();                           // vytvoreni menu s vyberem telesa
    glutTimerFunc(10, onTimer, 0x1234);     // registrace funkce volane pri tiku casovace
    glutMainLoop();                         // nekonecna smycka, kde se volaji zaregistrovane funkce
    return 0;                               // navratova hodnota vracena operacnimu systemu (unreachable - nutno kvuli ANSI)
}

