#include "utils.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(_WIN32)
#include <sys/timeb.h>
#else
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/times.h>
#endif

#ifndef CLK_TCK
#define CLK_TCK 1000
#endif

PPMImage *image;

//---------------------------------------------------------------------
// Tato funkce vytvori menu a navaze ho na leve tlacitko mysi
//---------------------------------------------------------------------
void createMenu(void)
{
    glutCreateMenu(onCommand);              // vytvoreni menu    
    glutAddMenuEntry("NEW GAME",0);        
    glutAddMenuEntry("fill/lines",10);    
    glutAddMenuEntry("textures",11);    
    glutAddMenuEntry("idspispopd",12);    
    glutAddMenuEntry("showfps",54);        
    glutAddMenuEntry("fullscreen",55);    
    glutAddMenuEntry("EXIT",80);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
}

//---------------------------------------------------------------------
// Tato funkce je zavolana pri vyberu polozky z menu
//---------------------------------------------------------------------
void onCommand(int command)
{
  // info.mouseCheck=false;
    switch (command) {
        case 0:
          initNewGame();
          break;
        case 10: // line
            info.filled=!info.filled;
            if (info.filled) {
              glPolygonMode(GL_FRONT,GL_FILL);
              glPolygonMode(GL_BACK,GL_FILL);
            } else {
              glPolygonMode(GL_FRONT,GL_LINE);
              glPolygonMode(GL_BACK,GL_LINE);
            }
            break;
        case 11:
            info.textured=!info.textured;
            break;
        case 12:
            info.clip=!info.clip;
            break;
        case 80:                            // ukonceni aplikace
            exit(0);
        case 54:
             info.showfps=!info.showfps;
             break;
        case 55:
              toggleFullScreen();
              break;
        default:
            break;
    }
    glutWarpPointer(info.windowCenterX,info.windowCenterY);
    glutPostRedisplay();                    // prekreslit scenu
    // info.mouseCheck=true;
}

void toggleFullScreen(void) {
  info.mouseCheck=false;
  if (info.fullscreen) {
    glutReshapeWindow(500,500);    // prepnuti zpet do okna
    glutPositionWindow(50,50);
    glutPostRedisplay();
    info.windowCenterX=glutGet(GLUT_WINDOW_WIDTH)/2;
    info.windowCenterY=glutGet(GLUT_WINDOW_HEIGHT)/2;
    glutWarpPointer(info.windowCenterX,info.windowCenterY);
    info.fullscreen=false;
  } else {
    glutFullScreen();               // prepnuti na celou obrazovku                   
    glutPostRedisplay();
    info.windowCenterX=glutGet(GLUT_WINDOW_WIDTH)/2;
    info.windowCenterY=glutGet(GLUT_WINDOW_HEIGHT)/2;
    glutWarpPointer(info.windowCenterX,info.windowCenterY);
    info.fullscreen=true;
  }
  info.mouseCheck=true;
}

float elapsed(void)
{
    static long begin = 0;
    static long finish, difference;

#if defined(_WIN32)
    static struct timeb tb;
    ftime(&tb);
    finish = tb.time*1000+tb.millitm;
#else
    static struct tms tb;
    finish = times(&tb);
#endif

    difference = finish - begin;
    begin = finish;

    return (float)difference/(float)CLK_TCK;
}

//---------------------------------------------------------------------
// Tato funkce vypise text na 2d
//---------------------------------------------------------------------
void printText(int x, int y, char* s) 
{
    int lines;
    char* p;

    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0, glutGet(GLUT_WINDOW_WIDTH), 
	    0, glutGet(GLUT_WINDOW_HEIGHT), -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
    glColor3ub(0, 128, 255);
    glRasterPos2i(x, y);
    for(p = s, lines = 0; *p; p++) {
	if (*p == '\n') {
	    lines++;
	    glRasterPos2i(x, y-(lines*18));
	}
	glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *p);
    }
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glEnable(GL_DEPTH_TEST);
}

//------------------------------------------------------------------------------

TClovek::TClovek()
{
  mapaOut.x1=250;
  mapaOut.x2=-250;
  mapaOut.y1=250;
  mapaOut.y2=-250;
  mapaIn.x1=-88;
  mapaIn.x2=92;
  mapaIn.y1=107;  
  mapaIn.y2=245;
  setSpeed();
  setSensitivity();
  setDaleko();
  accel=3;
  posX=0;
  posY=0;
  posZ=8;
  posAlfa=0;
  posBeta=0;
  fov=45;
  nearPlane=0.1;
  farPlane=1500;
  posXlast=posX;
  posYlast=posY;
  prepocitejAll();
}

void TClovek::prepocitejAll()
{
  if (info.clip) {
    if (posX>mapaOut.x1) posX=mapaOut.x1;
    if (posX<mapaOut.x2) posX=mapaOut.x2;  
    if (posY>mapaOut.y1) posY=mapaOut.y1;
    if (posY<mapaOut.y2) posY=mapaOut.y2;
    
    if (posXlast<=mapaIn.x1 || posXlast>=mapaIn.x2) {    
      if (posY>mapaIn.y1 && posY<mapaIn.y2) {
        GLdouble stred=(mapaIn.x2-mapaIn.x1)/2+mapaIn.x1;
        if (posX>mapaIn.x1 && posX<stred) posX=mapaIn.x1;
        else if (posX<mapaIn.x2 && (posX>(mapaIn.x2-mapaIn.x1)/2+mapaIn.x1)) posX=mapaIn.x2;
      }
    }
    if (posYlast<=mapaIn.y1 || posYlast>=mapaIn.y2) {
      if (posX>mapaIn.x1 && posX<mapaIn.x2) {
        GLdouble stred=(mapaIn.y2-mapaIn.y1)/2+mapaIn.y1;
        if (posY>mapaIn.y1 && posY<stred) posY=mapaIn.y1;
        else if (posY<mapaIn.y2 && posY>stred) posY=mapaIn.y2;
      }
    }
    
  }  
  eyeX=posX;
  eyeY=posY;
  eyeZ=posZ;
  centerX=sin(posAlfa)*daleko+posX;
  centerY=(cos(posAlfa)*cos(posBeta))*daleko+posY;
  centerZ=sin(posBeta)*daleko+posZ;
  posAlfaD=posAlfa*180/PI;
  posBetaD=posBeta*180/PI;
  posXlast=posX;
  posYlast=posY;
}

bool TClovek::jdiY(bool smer, bool acc)
{
  GLdouble spd=speed;
  if (acc) spd=speed*accel;
  if (!smer) {
    posX=posX-sin(posAlfa)*spd;
    posY=posY-cos(posAlfa)*spd;    
  } else {
    posX=posX+sin(posAlfa)*spd;
    posY=posY+cos(posAlfa)*spd;    
  }
  prepocitejAll();
}
bool TClovek::jdiX(bool smer,bool acc)
{
  GLdouble tmp=posAlfa+PI/2;
  GLdouble spd=speed;
  if (acc) spd=speed*accel;
  while (tmp>=2*PI) tmp=tmp-2*PI;
  while (tmp<0) tmp=tmp+2*PI;
  if (smer) {
    posX=posX-sin(tmp)*spd;
    posY=posY-cos(tmp)*spd;    
  } else {
    posX=posX+sin(tmp)*spd;
    posY=posY+cos(tmp)*spd;    
  }
  prepocitejAll();    
}
bool TClovek::jdiZ(bool smer)
{
  if (smer) posZ=8;
    else posZ=-2;
  prepocitejAll();    
}
void TClovek::otocX(bool smer,int kolik)
{
  if (!smer) {
    posAlfa=posAlfa+sensi*kolik;
  } else {
    posAlfa=posAlfa-sensi*kolik;
  }
  while (posAlfa>=2*PI) posAlfa=posAlfa-2*PI;
  while (posAlfa<0) posAlfa=posAlfa+2*PI;
  prepocitejAll();
}
bool TClovek::otocZ(bool smer,int kolik)
{
  if (smer) {
    posBeta=posBeta+sensi*kolik;
    if (posBeta>=PI/5) posBeta=PI/5;
  } else {
    posBeta=posBeta-sensi*kolik;
    if (posBeta<=-PI/5) posBeta=-PI/5;
  }
  prepocitejAll();
}

//------------------------------------------------------------------------------
void TZbran::kresli()
{
  if (visible) {
    glTranslatef(1.8,10.7,(-2.9));    
    glmDraw(point,GLM_SMOOTH | GLM_TEXTURE);              
  }
}

TZbran::TZbran()
{
  visible=false;
  point=glmReadOBJ("data/bambitka.obj");
  glmScale(point,0.5);
  glmFacetNormals(point);
  glmVertexNormals(point,90);
}

TZbran::~TZbran()
{
  if (point) delete point;
  point=NULL;
}
//------------------------------------------------------------------------------
void TTerc::kresli(bool stoji)
{ 
  if (!stoji) 
  {
    glTranslatef(0,8,-5.8);            
    glRotatef(-90.0,1.0,0.0,0.0);
  }
  glmDraw(point,GLM_SMOOTH | GLM_TEXTURE);              
  if (!stoji) {
    glRotatef(90.0,1.0,0.0,0.0);
    glTranslatef(0,-8,5.8);            
  }
}

TTerc::TTerc()
{
  point=glmReadOBJ("data/terc.obj");
  glmScale(point,0.5);
  glmFacetNormals(point);
  glmVertexNormals(point,90);
}

TTerc::~TTerc()
{
  if (point) delete point;
  point=NULL;
}


//------------------------------------------------------------------------------
void TStrelnice::kresli()
{
  glTranslatef(0,0,0);
  glmDraw(point,GLM_SMOOTH | GLM_TEXTURE);              
}

TStrelnice::TStrelnice()
{
  point=glmReadOBJ("data/strelnice.obj");
  glmScale(point,0.5);
  glmFacetNormals(point);
  glmVertexNormals(point,90);
}

TStrelnice::~TStrelnice()
{
  if (point) delete point;
  point=NULL;
}

//------------------------------------------------------------------------------

void initTextury() {
  image=LoadPPM("data/strelnice2.ppm");
  // image=LoadBMP("data/strelnice2.bmp");
  if (!image) {
    printf("\nChyba pri nacitani dat (malo pameti nebo data chybi)\n");
    exit(1);
  }
  glPixelStorei(GL_UNPACK_ALIGNMENT,1);      // mod ulozeni pixelu
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); // opakovani textury
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST); // volba filtru
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
  gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,image->sizeX,image->sizeY,GL_RGB,GL_UNSIGNED_BYTE,image->data); // nacteni textury do GPU
}

int rnd(int range)
{
  int r;
  r=rand()%range;
  return(r);
}

void initNewGame(int pocet)
{
  int je=0;
  if (pocet>=18) pocet=18;
  for (int ii=0;ii<3;ii++)
    for (int i=0;i<6;i++)
      info.terce[ii][i]=0;
      
  while (je<pocet) {
  for (int ii=0;ii<3;ii++)
    for (int i=0;i<6;i++)
      if (rnd(10)<1 && !info.terce[ii][i] && je<pocet) {
        info.terce[ii][i]=1;
        je=je+1;
      }
  }
}
